'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.markUnatched = exports.markWatched = exports.fetchMovies = exports.fetchMediaGUID = exports.fetchMedia = undefined;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _xml2js = require('xml2js');

var _isomorphicFetch = require('isomorphic-fetch');

var _isomorphicFetch2 = _interopRequireDefault(_isomorphicFetch);

var _rateLimit = require('./rateLimit');

var _ui = require('./ui');

require('./env');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var PAGE_SIZE = 32;

var parseXML = function parseXML(xml) {
  return new _promise2.default(function (resolve, reject) {
    (0, _xml2js.parseString)(xml, function (err, data) {
      if (err) reject(err);else resolve(data);
    });
  });
};

var fetchText = function fetchText(url) {
  return (0, _isomorphicFetch2.default)(url).then(function (res) {
    return res.text();
  });
};

var fetchXML = function fetchXML(url) {
  return fetchText(url).then(function (res) {
    return parseXML(res);
  });
};

var rateLimitFetchXML = function rateLimitFetchXML(url) {
  return (0, _rateLimit.concurrent)(fetchXML, url);
};

var flatten = function flatten(list) {
  return list.reduce(function (a, b) {
    return a.concat(Array.isArray(b) ? flatten(b) : b);
  }, []);
};

var fetchMediaContainer = function () {
  var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(server) {
    var page = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;
    var start, url;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            start = (page - 1) * PAGE_SIZE;
            url = server.protocol + '://' + server.host + '/library/sections/' + server.section + '/allLeaves?X-Plex-Token=' + server.token + '&X-Plex-Container-Start=' + start + '&X-Plex-Container-Size=' + PAGE_SIZE;
            _context.next = 4;
            return rateLimitFetchXML(url);

          case 4:
            return _context.abrupt('return', _context.sent);

          case 5:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined);
  }));

  return function fetchMediaContainer(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

var fetchMedia = exports.fetchMedia = function () {
  var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(server) {
    var totalSize, totalPages, promises, i, videos;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return fetchMediaContainer(server);

          case 2:
            _context2.t0 = _context2.sent.MediaContainer.$.totalSize;
            totalSize = parseInt(_context2.t0, 10);
            totalPages = Math.ceil(totalSize / PAGE_SIZE);

            // Fetch all videos

            promises = [];

            for (i = 1; i <= totalPages; i += 1) {
              promises.push(fetchMediaContainer(server, i));
            }

            // Unpack video results
            videos = [];
            _context2.next = 10;
            return _promise2.default.all(promises).then(function (results) {
              results.forEach(function (res) {
                res.MediaContainer.Video.forEach(function (video) {
                  videos.push(video);
                });
              });
            });

          case 10:
            return _context2.abrupt('return', videos.map(function (media) {
              return media.$;
            }).map(function (_ref3) {
              var _ref3$grandparentTitl = _ref3.grandparentTitle;
              var grandparentTitle = _ref3$grandparentTitl === undefined ? '' : _ref3$grandparentTitl;
              var _ref3$index = _ref3.index;
              var index = _ref3$index === undefined ? '' : _ref3$index;
              var key = _ref3.key;
              var _ref3$parentIndex = _ref3.parentIndex;
              var parentIndex = _ref3$parentIndex === undefined ? '' : _ref3$parentIndex;
              var title = _ref3.title;
              var viewCount = _ref3.viewCount;
              var year = _ref3.year;
              return {
                guid: grandparentTitle + ' - ' + parentIndex + ' - ' + index + ' - ' + year + ' - ' + title,
                key: key,
                title: title,
                watched: parseInt(viewCount || '0', 10) > 0,
                year: year
              };
            }));

          case 11:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, undefined);
  }));

  return function fetchMedia(_x4) {
    return _ref2.apply(this, arguments);
  };
}();

var fetchMediaGUID = exports.fetchMediaGUID = function () {
  var _ref4 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3(server, media) {
    var url;
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            url = server.protocol + '://' + server.host + media.key + '?X-Plex-Token=' + server.token;
            _context3.next = 3;
            return rateLimitFetchXML(url).then(function (res) {
              return res.MediaContainer.Video[0].$;
            }).then(function (_ref5) {
              var guid = _ref5.guid;
              return (0, _extends3.default)({}, media, {
                guid: guid
              });
            });

          case 3:
            return _context3.abrupt('return', _context3.sent);

          case 4:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, undefined);
  }));

  return function fetchMediaGUID(_x5, _x6) {
    return _ref4.apply(this, arguments);
  };
}();

var fetchMovies = exports.fetchMovies = function () {
  var _ref6 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee4(server) {
    var fuzzy = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
    var media;
    return _regenerator2.default.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            _context4.next = 2;
            return fetchMedia(server);

          case 2:
            media = _context4.sent;

            if (!fuzzy) {
              _context4.next = 7;
              break;
            }

            _context4.t0 = media;
            _context4.next = 10;
            break;

          case 7:
            _context4.next = 9;
            return (0, _ui.progressMap)(media, function (movie) {
              return fetchMediaGUID(server, movie);
            });

          case 9:
            _context4.t0 = _context4.sent;

          case 10:
            return _context4.abrupt('return', _context4.t0);

          case 11:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, undefined);
  }));

  return function fetchMovies(_x7, _x8) {
    return _ref6.apply(this, arguments);
  };
}();

var extractID = function extractID(key) {
  return key.match(/\/library\/metadata\/(\d+)/)[1];
};

var markWatched = exports.markWatched = function () {
  var _ref7 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee5(server, movie) {
    return _regenerator2.default.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            _context5.next = 2;
            return fetchText(server.protocol + '://' + server.host + '/:/scrobble?identifier=com.plexapp.plugins.library&key=' + extractID(movie.key) + '&X-Plex-Token=' + server.token);

          case 2:
            return _context5.abrupt('return', _context5.sent);

          case 3:
          case 'end':
            return _context5.stop();
        }
      }
    }, _callee5, undefined);
  }));

  return function markWatched(_x10, _x11) {
    return _ref7.apply(this, arguments);
  };
}();

var markUnatched = exports.markUnatched = function () {
  var _ref8 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee6(server, movie) {
    return _regenerator2.default.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            _context6.next = 2;
            return fetchText(server.protocol + '://' + server.host + '/:/unscrobble?identifier=com.plexapp.plugins.library&key=' + extractID(movie.key) + '&X-Plex-Token=' + server.token);

          case 2:
            return _context6.abrupt('return', _context6.sent);

          case 3:
          case 'end':
            return _context6.stop();
        }
      }
    }, _callee6, undefined);
  }));

  return function markUnatched(_x12, _x13) {
    return _ref8.apply(this, arguments);
  };
}();