'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parseCLIArg = exports.progressMap = exports.exitToken = exports.exitUsage = exports.TOKEN = exports.log = undefined;

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _tsProgress = require('ts-progress');

var _tsProgress2 = _interopRequireDefault(_tsProgress);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var log = exports.log = function log() {
  var _console;

  return (_console = console).error.apply(_console, arguments);
}; /* eslint-disable no-console */

var TOKEN = exports.TOKEN = process.env.PLEX_TOKEN;

var exitUsage = exports.exitUsage = function exitUsage() {
  console.error('\nUsage: plex-sync [https://][token@]IP[:PORT]/SECTION[,rw] [https://][token@]IP[:PORT]/SECTION[,rw]\n                 [[https://][token@]IP[:PORT]/SECTION[,rw]...]\n\nExample:\n\n    Sync section 1 on a server with the default port with section 2 on another server:\n    $ plex-sync 10.0.1.2/1 10.0.1.3:32401/2\n\n    Sync three servers:\n    $ plex-sync 10.0.1.2/1 10.0.1.3/1 10.0.1.4/1\n\n    Sync with a server via HTTPS\n    $ plex-sync 10.0.1.2/2 https://server-domain/3\n\n    Dry run, to see what the script will do:\n    $ DRY_RUN=1 plex-sync 10.0.1.5/1 10.0.1.5/1\n\n    Precise matching (slow and may crash the Plex server):\n    $ MATCH_TYPE=precise plex-sync 10.0.1.5/1 10.0.1.5/1\n\n    Syncing between multiple Plex users (different access tokens):\n    $ plex-sync xxxxxx@10.0.1.5/1 zzzzzz@10.0.1.10/3\n\n    Unidirectional sync (read from one server, write to the other):\n    $ plex-sync 10.0.1.5/1,r 10.0.1.10/3,w\n\n    Complex use case:\n    $ plex-sync xxxx@10.0.1.5:32401/1,r https://yyyy@10.0.1.10/3,w zzzz@10.0.1.15/2,rw\n'.trim());
  process.exit(1);
};

var exitToken = exports.exitToken = function exitToken() {
  console.error('\nError: missing Plex authentication token for one or more of the specified servers\n\nPlease either set your Plex authentication token via the PLEX_TOKEN environment variable\nor pass the Plex token in the server definition (TOKEN@0.0.0.0...).  See \'plex-sync help\'\nfor more information.\n\nIf you need help locating your Plex authentication token, feel free to use the bookmarklet\nlocated at https://jacobwgillespie.github.io/plex-token-bookmarklet/\n'.trim());
  process.exit(1);
};

var progressMap = exports.progressMap = function progressMap(items, fn) {
  var disable = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

  var progress = disable ? null : _tsProgress2.default.create({ total: items.length });

  return _promise2.default.all(items.map(function () {
    if (!disable) progress.update();
    return fn.apply(undefined, arguments);
  }));
};

var parseCLIArg = exports.parseCLIArg = function parseCLIArg(arg) {
  var matches = arg.match(/^((https?):\/\/)?(([^@]+)@)?(([^:]+)(:\d+)?)\/(\d+)(,[rw][rw]?)?$/);
  if (!matches) exitUsage();
  var protocol = matches[2] === 'https' ? 'https' : 'http';
  var token = matches[4] || TOKEN;
  var host = '' + matches[6] + (matches[7] || ':32400');
  var section = matches[8] || '1';
  var modeString = matches[10] || 'rw';
  var mode = {
    read: modeString.includes('r'),
    write: modeString.includes('w')
  };

  if (!token) exitToken();

  return { protocol: protocol, token: token, host: host, section: section, mode: mode };
};